@@ -12,14 +12,24 @@
  * @{
  */
 
-#include <netlink-private/netlink.h>
-#include <netlink-private/tc.h>
+#include "nl-default.h"
+
 #include <netlink/netlink.h>
-#include <netlink-private/route/tc-api.h>
 #include <netlink/route/classifier.h>
 #include <netlink/route/cls/fw.h>
 
+#include "tc-api.h"
+
 /** @cond SKIP */
+struct rtnl_fw {
+	uint32_t cf_classid;
+	struct nl_data *cf_act;
+	struct nl_data *cf_police;
+	char cf_indev[IFNAMSIZ];
+	uint32_t cf_fwmask;
+	int cf_mask;
+};
+
 #define FW_ATTR_CLASSID      0x001
 #define FW_ATTR_ACTION       0x002
 #define FW_ATTR_POLICE       0x004
@@ -203,12 +213,12 @@ static struct rtnl_tc_ops fw_ops = {
 	},
 };
 
-static void __init fw_init(void)
+static void _nl_init fw_init(void)
 {
 	rtnl_tc_register(&fw_ops);
 }
 
-static void __exit fw_exit(void)
+static void _nl_exit fw_exit(void)
 {
 	rtnl_tc_unregister(&fw_ops);
 }
